@@ -7,8 +7,13 @@ namespace FSpot
 {
 	public class Preferences
 	{
-		public const string MAIN_WINDOW_MAXIMIZED = "/apps/f-spot/ui/maximized";
+		public const string APP_FSPOT = "/apps/f-spot/";
+		public const string APP_FSPOT_EXPORT = APP_FSPOT + "export/";
+		public const string APP_FSPOT_EXPORT_TOKENS = APP_FSPOT_EXPORT + "tokens/";
 
+		public const string GTK_RC = "/apps/f-spot/ui/gtkrc";
+
+		public const string MAIN_WINDOW_MAXIMIZED = "/apps/f-spot/ui/maximized";
 		public const string MAIN_WINDOW_X = "/apps/f-spot/ui/main_window_x";
 		public const string MAIN_WINDOW_Y = "/apps/f-spot/ui/main_window_y";
 		public const string MAIN_WINDOW_WIDTH = "/apps/f-spot/ui/main_window_width";
@@ -26,13 +31,21 @@ namespace FSpot
 		public const string VIEWER_INTERPOLATION = "/apps/f-spot/viewer/interpolation";
 		public const string VIEWER_TRANS_COLOR = "/apps/f-spot/viewer/trans_color";
 		public const string VIEWER_TRANSPARENCY = "/apps/f-spot/viewer/transparency";
+		public const string CUSTOM_CROP_RATIOS = "/apps/f-spot/viewer/custom_crop_ratios";
+		
+		public const string COLOR_MANAGEMENT_ENABLED = "/apps/f-spot/ui/color_management_enabled";
+		public const string COLOR_MANAGEMENT_USE_X_PROFILE = "/apps/f-spot/ui/color_management_use_x_profile";
+		public const string COLOR_MANAGEMENT_DISPLAY_PROFILE = "/apps/f-spot/ui/color_management_display_profile";
+		public const string COLOR_MANAGEMENT_OUTPUT_PROFILE = "/apps/f-spot/ui/color_management_output_profile";
 		
 		public const string SHOW_TOOLBAR = "/apps/f-spot/ui/show_toolbar";
 		public const string SHOW_SIDEBAR = "/apps/f-spot/ui/show_sidebar";
 		public const string SHOW_TIMELINE = "/apps/f-spot/ui/show_timeline";
+		public const string SHOW_FILMSTRIP = "/apps/f-spot/ui/show_filmstrip";
 		public const string SHOW_TAGS = "/apps/f-spot/ui/show_tags";
 		public const string SHOW_DATES = "/apps/f-spot/ui/show_dates";
 		public const string EXPANDED_TAGS = "/apps/f-spot/ui/expanded_tags";
+		public const string SHOW_RATINGS = "/apps/f-spot/ui/show_ratings";
 		public const string TAG_ICON_SIZE = "/apps/f-spot/ui/tag_icon_size";
 		
 		public const string GLASS_POSITION = "/apps/f-spot/ui/glass_position";
@@ -42,46 +55,6 @@ namespace FSpot
 		public const string SIDEBAR_POSITION = "/apps/f-spot/ui/sidebar_size";
 		public const string ZOOM = "/apps/f-spot/ui/zoom";
 
-		public const string EXPORT_FLICKR_SCALE = "/apps/f-spot/export/flickr/scale";
-		public const string EXPORT_FLICKR_SIZE = "/apps/f-spot/export/flickr/size";
-		public const string EXPORT_FLICKR_BROWSER = "/apps/f-spot/export/flickr/browser";
-		public const string EXPORT_FLICKR_TAGS = "/apps/f-spot/export/flickr/tags";
-		public const string EXPORT_FLICKR_STRIP_META = "/apps/f-spot/export/flickr/strip_meta";
-		public const string EXPORT_FLICKR_EMAIL = "/apps/f-spot/export/flickr/email";
-		public const string EXPORT_FLICKR_PUBLIC = "/apps/f-spot/export/flickr/public";
-		public const string EXPORT_FLICKR_FRIENDS = "/apps/f-spot/export/flickr/friends";
-		public const string EXPORT_FLICKR_FAMILY = "/apps/f-spot/export/flickr/family";
-
-		public const string EXPORT_TOKEN_FLICKR = "/apps/f-spot/export/tokens/flickr"; 
-		public const string EXPORT_TOKEN_23HQ = "/apps/f-spot/export/tokens/23hq"; 
-		public const string EXPORT_TOKEN_ZOOOMR = "/apps/f-spot/export/tokens/zooomr"; 
-
-		public const string EXPORT_GALLERY_SCALE = "/apps/f-spot/export/gallery/scale";
-		public const string EXPORT_GALLERY_SIZE = "/apps/f-spot/export/gallery/size";
-		public const string EXPORT_GALLERY_BROWSER = "/apps/f-spot/export/gallery/browser";
-		public const string EXPORT_GALLERY_META = "/apps/f-spot/export/gallery/meta";
-		public const string EXPORT_GALLERY_ROTATE = "/apps/f-spot/export/gallery/rotate";
-
-		public const string EXPORT_PICASAWEB_SCALE = "/apps/f-spot/export/picasaweb/scale";
-		public const string EXPORT_PICASAWEB_SIZE = "/apps/f-spot/export/picasaweb/size";
-		public const string EXPORT_PICASAWEB_ROTATE = "/apps/f-spot/export/picasaweb/rotate";
-		public const string EXPORT_PICASAWEB_BROWSER = "/apps/f-spot/export/picasaweb/browser";
-		public const string EXPORT_PICASAWEB_TAG = "/apps/f-spot/export/picasaweb/tag";
-
-		public const string EXPORT_SMUGMUG_SCALE = "/apps/f-spot/export/smugmug/scale";
-		public const string EXPORT_SMUGMUG_SIZE = "/apps/f-spot/export/smugmug/size";
-		public const string EXPORT_SMUGMUG_ROTATE = "/apps/f-spot/export/smugmug/rotate";
-		public const string EXPORT_SMUGMUG_BROWSER = "/apps/f-spot/export/smugmug/browser";
-
-		public const string EXPORT_FOLDER_SCALE = "/apps/f-spot/export/folder/scale";
-		public const string EXPORT_FOLDER_SIZE = "/apps/f-spot/export/folder/size";
-		public const string EXPORT_FOLDER_OPEN = "/apps/f-spot/export/folder/browser";
-		public const string EXPORT_FOLDER_ROTATE = "/apps/f-spot/export/folder/rotate";
-		public const string EXPORT_FOLDER_METHOD = "/apps/f-spot/export/folder/method";
-		public const string EXPORT_FOLDER_URI = "/apps/f-spot/export/folder/uri";
-		public const string EXPORT_FOLDER_SHARPEN = "/apps/f-spot/export/folder/sharpen";
-		public const string EXPORT_FOLDER_INCLUDE_TARBALLS = "/apps/f-spot/export/folder/include_tarballs";
-		
 		public const string EXPORT_EMAIL_SIZE = "/apps/f-spot/export/email/size";
 		public const string EXPORT_EMAIL_ROTATE = "/apps/f-spot/export/email/auto_rotate";
 		public const string EXPORT_EMAIL_DELETE_TIMEOUT_SEC = "/apps/f-spot/export/email/delete_timeout_seconds";
@@ -109,37 +82,45 @@ namespace FSpot
 		public const string PROXY_PASSWORD = "/system/http_proxy/authentication_password";
 		public const string PROXY_BYPASS_LIST = "/system/http_proxy/ignore_hosts";
 
+		public const string GSD_THUMBS_MAX_AGE = "/desktop/gnome/thumbnail_cache/maximum_age";
+		public const string GSD_THUMBS_MAX_SIZE = "/desktop/gnome/thumbnail_cache/maximum_size";
 
-		private static GConf.Client client;
-		private static GConf.NotifyEventHandler changed_handler;
-		private static Dictionary<string, object> cache = new Dictionary<string, object>();
 
-		private static GConf.Client Client 
-		{
+		private static IPreferenceBackend backend;
+		private static NotifyChangedHandler changed_handler;
+		private static IPreferenceBackend Backend {
 			get {
-				if (client == null) {
-					client = new GConf.Client ();
-
-					changed_handler = new GConf.NotifyEventHandler (OnSettingChanged);
-					client.AddNotify ("/apps/f-spot", changed_handler);
-					client.AddNotify ("/apps/gnome-screensaver/themes", changed_handler);
-					client.AddNotify ("/apps/gnome-screensaver/mode", changed_handler);
-					client.AddNotify ("/desktop/gnome/url-handlers/mailto", changed_handler);
-					client.AddNotify ("/system/http_proxy", changed_handler);
+				if (backend == null) {
+#if !NOGCONF
+					try {
+						backend = new GConfPreferenceBackend ();
+					} catch (Exception ex) {
+						Console.WriteLine ("Couldn't load Gconf. Check that gconf-daemon is running.{0}{1}",
+							Environment.NewLine, ex);
+						backend = new NullPreferenceBackend ();
+					}
+#else
+					backend = new NullPreferenceBackend ();
+#endif
+					changed_handler = new NotifyChangedHandler (OnSettingChanged);
+					backend.AddNotify ("/apps/f-spot", changed_handler);
+					backend.AddNotify ("/apps/gnome-screensaver/themes", changed_handler);
+					backend.AddNotify ("/apps/gnome-screensaver/mode", changed_handler);
+					backend.AddNotify ("/desktop/gnome/url-handlers/mailto", changed_handler);
+					backend.AddNotify ("/system/http_proxy", changed_handler);
 				}
-				return client;
+				return backend;
 			}
 		}
+		private static Dictionary<string, object> cache = new Dictionary<string, object>();
 
-		public static object GetDefault (string key)
+		static object GetDefault (string key)
 		{
 			switch (key) {
 			case MAIN_WINDOW_X:
 			case MAIN_WINDOW_Y:
 			case MAIN_WINDOW_HEIGHT:
 			case MAIN_WINDOW_WIDTH:
-				return null;
-				
 			case IMPORT_WINDOW_HEIGHT:
 			case IMPORT_WINDOW_WIDTH:
 			case IMPORT_WINDOW_PANE_POSITION:
@@ -157,29 +138,15 @@ namespace FSpot
 			case SHOW_TOOLBAR:
 			case SHOW_SIDEBAR:
 			case SHOW_TIMELINE:
+			case SHOW_FILMSTRIP:
 			case SHOW_TAGS:
 			case SHOW_DATES:
+			case SHOW_RATINGS:
 			case VIEWER_SHOW_FILENAMES:
-			case EXPORT_GALLERY_ROTATE:
-			case EXPORT_PICASAWEB_SCALE:
-			case EXPORT_PICASAWEB_ROTATE:
-			case EXPORT_PICASAWEB_BROWSER:
-			case EXPORT_PICASAWEB_TAG:
-			case EXPORT_SMUGMUG_SCALE:
-			case EXPORT_SMUGMUG_ROTATE:
-			case EXPORT_SMUGMUG_BROWSER:
-			case EXPORT_FOLDER_SCALE:
-			case EXPORT_FOLDER_SHARPEN:
-			case EXPORT_FOLDER_INCLUDE_TARBALLS:
 				return true;
 			
-			case EXPORT_PICASAWEB_SIZE:
-			case EXPORT_SMUGMUG_SIZE:
-			case EXPORT_FOLDER_SIZE:
-				return 800;
-				
 			case TAG_ICON_SIZE:
-				return (int) Tag.IconSize.Large;
+				return (int) Tag.IconSize.Medium;
 		
 			case SIDEBAR_POSITION:
 			case ZOOM:
@@ -195,14 +162,10 @@ namespace FSpot
 			case EXPORT_EMAIL_SIZE:
 				return 3;	// medium size 640px
 			case EXPORT_EMAIL_ROTATE:
-			case EXPORT_FOLDER_OPEN:
-			case EXPORT_FOLDER_ROTATE:
 			case VIEWER_INTERPOLATION:
 				return true;
 			case EXPORT_EMAIL_DELETE_TIMEOUT_SEC:
 				return 30;	// delete temporary email pictures after 30 seconds
-			case EXPORT_FOLDER_METHOD:
-				return "static";
 			case VIEWER_TRANSPARENCY:
 				return "NONE";
 			case VIEWER_TRANS_COLOR:
@@ -216,13 +179,14 @@ namespace FSpot
 				return 0;
 			case PROXY_USER:
 			case PROXY_PASSWORD:
+			case GTK_RC:
 				return String.Empty;
-			
 			default:
 				return null;
 			}
 		}
 		
+		[Obsolete ("use Get<T> (string key) instead")]
 		public static object Get (string key)
 		{
 			lock (cache) {
@@ -231,16 +195,47 @@ namespace FSpot
 					return val;
 
 				try {
-					val = Client.Get (key);
-					cache.Add (key, val);
-				} catch (GConf.NoSuchKeyException) {
+					val = Backend.Get (key);
+				} catch (NoSuchKeyException) {
 					val = GetDefault (key);
+				}
+				
+				cache.Add (key, val);
+				return val;
+			}
+		}
 
-					if (val != null)
-						Set (key, val);
+		//return true if the key exists in the backend
+		public static bool TryGet<T> (string key, out T value)
+		{
+			lock (cache) {
+				value = default (T);
+				object o;
+				if (cache.TryGetValue (key, out o)) {
+					value = (T)o;
+					return true;
 				}
 
+				try {
+					value = (T) Backend.Get (key);
+				} catch { //catching NoSuchKeyException
+					return false;
+				}
+				
+				cache.Add (key, value);
+				return true;
+			}
+		}
+
+		public static T Get<T> (string key)
+		{
+			T val;
+			if (TryGet<T> (key, out val))
 				return val;
+			try {
+				return (T) GetDefault (key);
+			} catch { //catching InvalidCastException
+				return default (T);
 			}
 		}
 
@@ -249,26 +244,17 @@ namespace FSpot
 			lock (cache) {
 				try {
 					cache [key] = value;				
-					Client.Set (key, value);
-				} catch {
-					Console.WriteLine ("Unable to write this gconf key :"+key);
+					Backend.Set (key, value);
+				} catch (Exception e){
+					Console.WriteLine (e);
+					Console.WriteLine ("Unable to set this :"+key);
 				}
 			}
 		}
 
-		public static void SetAsBackground (string path)
-		{
-			Client.Set ("/desktop/gnome/background/color_shading_type", "solid");
-			Client.Set ("/desktop/gnome/background/primary_color", "#000000");
-			Client.Set ("/desktop/gnome/background/picture_options", "stretched");
-			Client.Set ("/desktop/gnome/background/picture_opacity", 100);
-			Client.Set ("/desktop/gnome/background/picture_filename", path);
-			Client.Set ("/desktop/gnome/background/draw_background", true);
-		}
+		public static event NotifyChangedHandler SettingChanged;
 
-		public static event GConf.NotifyEventHandler SettingChanged;
-
-		static void OnSettingChanged (object sender, GConf.NotifyEventArgs args)
+		static void OnSettingChanged (object sender, NotifyEventArgs args)
 		{
 			lock (cache) {
 				if (cache.ContainsKey (args.Key)) {
@@ -285,20 +271,20 @@ namespace FSpot
 		{
 			WebProxy proxy = null;
 			
-			if ((bool) Preferences.Get (PROXY_USE_PROXY))
+			if (Preferences.Get<bool> (PROXY_USE_PROXY))
 				return null;
 
 			try {
 				string host;
 				int    port;
 				
-				host = (string) Preferences.Get (PROXY_HOST);
-				port = (int) Preferences.Get (PROXY_PORT);
+				host = Preferences.Get<string> (PROXY_HOST);
+				port = Preferences.Get<int> (PROXY_PORT);
 				
 				string uri = "http://" + host + ":" + port.ToString ();
 				proxy = new WebProxy (uri);
 
-				string [] bypass_list = (string []) Preferences.Get (PROXY_BYPASS_LIST);
+				string [] bypass_list = Preferences.Get<string[]> (PROXY_BYPASS_LIST);
 				if (bypass_list != null) {
 					for (int i = 0; i < bypass_list.Length; i++) {
 						bypass_list [i] = "http://" + bypass_list [i];
@@ -306,8 +292,8 @@ namespace FSpot
 					proxy.BypassList = bypass_list;
 				}
 
-				string username = (string) Preferences.Get (PROXY_USER);
-				string password = (string) Preferences.Get (PROXY_PASSWORD);
+				string username = Preferences.Get<string> (PROXY_USER);
+				string password = Preferences.Get<string> (PROXY_PASSWORD);
 
 				proxy.Credentials = new NetworkCredential (username, password);
 			} catch (Exception) {
