@@ -12,11 +12,15 @@ using Gtk;
 using Gnome;
 using System;
 
+using FSpot.Widgets;
 using FSpot.Filters;
+using FSpot.Utils;
+using FSpot.UI.Dialog;
+
 using Mono.Unix;
 
 namespace FSpot {
-	public class SendEmail : FSpot.GladeDialog {
+	public class SendEmail : GladeDialog {
 		PhotoQuery query;
 		Gtk.Window parent_window;
 
@@ -52,9 +56,11 @@ namespace FSpot {
 		{
 			this.selection = selection;
 
-			foreach (Photo p in selection.Items)
+			for (int i = 0; i < selection.Count; i++) {
+				Photo p = selection[i] as Photo;
 				if (Gnome.Vfs.MimeType.GetMimeTypeForUri (p.DefaultVersionUri.ToString ()) != "image/jpeg")
 					force_original = true;
+			}
 
 			if (force_original) {
 				original_size.Active = true;
@@ -64,7 +70,7 @@ namespace FSpot {
 				large_size.Sensitive = false;
 				x_large_size.Sensitive = false;
 			} else  
-				switch ((int) Preferences.Get (Preferences.EXPORT_EMAIL_SIZE)) {
+				switch (Preferences.Get<int> (Preferences.EXPORT_EMAIL_SIZE)) {
 					case 0 :  original_size.Active = true; break;
 					case 1 :  tiny_size.Active = true; break;
 					case 2 :  small_size.Active = true; break;
@@ -74,7 +80,7 @@ namespace FSpot {
 					default: break;
 				}
 
-			rotate_check.Active = (bool) Preferences.Get (Preferences.EXPORT_EMAIL_ROTATE);
+			rotate_check.Active = Preferences.Get<bool> (Preferences.EXPORT_EMAIL_ROTATE);
 			rotate_check.Sensitive = original_size.Active && tiny_size.Sensitive;
 			
 			tray_scrolled.Add (new TrayView (selection));
@@ -82,7 +88,8 @@ namespace FSpot {
 			Dialog.Modal = false;
 
 			// Calculate total original filesize 
-			foreach (Photo photo in selection.Items) {
+			for (int i = 0; i < selection.Count; i++) {
+				Photo photo = selection[i] as Photo;
 				try {
 					Orig_Photo_Size += (new Gnome.Vfs.FileInfo (photo.DefaultVersionUri.ToString ())).Size;
 				} catch {
@@ -94,7 +101,7 @@ namespace FSpot {
 
 
 			// Calculate approximate size shrinking, use first photo, and shrink to medium size as base.
-			Photo scalephoto = selection.Items [0] as Photo;			
+			Photo scalephoto = selection [0] as Photo;
 			if (scalephoto != null && !force_original) {
 				
 				// Get first photos file size
@@ -232,14 +239,14 @@ namespace FSpot {
 		
 			progress_dialog = new ProgressDialog (Catalog.GetString ("Preparing email"),
 							      ProgressDialog.CancelButtonType.Stop,
-							      selection.Items.Length, 
+							      selection.Count,
 							      parent_window);
 			
 			size = GetScaleSize(); // Which size should we scale to. 0 --> Original
 			
 			// evaluate mailto command and define attachment args for cli
 			System.Text.StringBuilder attach_arg = new System.Text.StringBuilder ();
-			switch (Preferences.Get (Preferences.GNOME_MAILTO_COMMAND) as string) {
+			switch (Preferences.Get<string> (Preferences.GNOME_MAILTO_COMMAND)) {
 				case "thunderbird %s":
 				case "mozilla-thunderbird %s":
 				case "seamonkey -mail -compose %s":
@@ -276,8 +283,8 @@ namespace FSpot {
 			filters.Add (new UniqueNameFilter (tmp_mail_dir));
 
 
-			foreach (Photo photo in selection.Items) {
-			
+			for (int i = 0; i < selection.Count; i++) {
+				Photo photo = selection [i] as Photo;
 				if ( (photo != null) && (!UserCancelled) ) {
 
 					if (progress_dialog != null)
@@ -287,16 +294,29 @@ namespace FSpot {
 					if (UserCancelled)
 					 	break;
 					 	
-					// Prepare a tmp_mail file name
-					FilterRequest request = new FilterRequest (photo.DefaultVersionUri);
+					try {
+						// Prepare a tmp_mail file name
+						FilterRequest request = new FilterRequest (photo.DefaultVersionUri);
 
-					filters.Convert (request);
-					request.Preserve(request.Current);
+						filters.Convert (request);
+						request.Preserve(request.Current);
 
- 					mail_attach.Append(attach_arg.ToString() + request.Current.ToString ());
-					
-					// Mark the path for deletion
-					tmp_paths.Add (request.Current.LocalPath);
+						mail_attach.Append(attach_arg.ToString() + request.Current.ToString ());
+						
+						// Mark the path for deletion
+						tmp_paths.Add (request.Current.LocalPath);
+					} catch (Exception e) {
+						Console.WriteLine("Error preparing {0}: {1}", selection[photo_index].Name, e.Message);
+						HigMessageDialog md = new HigMessageDialog (parent_window, 
+											    DialogFlags.DestroyWithParent,
+											    MessageType.Error,
+											    ButtonsType.Close,
+											    Catalog.GetString("Error processing image"), 
+											    String.Format(Catalog.GetString("An error occured while processing \"{0}\": {1}"), selection[photo_index].Name, e.Message));
+						md.Run();
+						md.Destroy();
+						UserCancelled = true;
+					}
 				}
 			} // foreach
 			
@@ -308,7 +328,7 @@ namespace FSpot {
 			else {		
 				// Send the mail :)
 				string mail_subject = Catalog.GetString("my photos");
-				switch (Preferences.Get (Preferences.GNOME_MAILTO_COMMAND) as string) {
+				switch (Preferences.Get<string> (Preferences.GNOME_MAILTO_COMMAND)) {
 					// openSuSE
 					case "thunderbird %s":
 						System.Diagnostics.Process.Start("thunderbird", " -compose \"subject=" + mail_subject + ",attachment='" + mail_attach + "'\"");
@@ -326,7 +346,7 @@ namespace FSpot {
 						System.Diagnostics.Process.Start("kmail", "  --composer --subject \"" + mail_subject + "\"" + mail_attach);
 					break;
 					default: 
-						GnomeUtil.UrlShow (parent_window,"mailto:?subject=" + System.Web.HttpUtility.UrlEncode(mail_subject) + mail_attach);
+						GnomeUtil.UrlShow ("mailto:?subject=" + System.Web.HttpUtility.UrlEncode(mail_subject) + mail_attach);
 					break;
 				}
 				                
@@ -334,7 +354,7 @@ namespace FSpot {
 				if (tmp_paths.Count > 0) {
 					// Fetch timeout value from preferences. In seconds. Needs to be multiplied with 1000 to get msec
 					uint delete_timeout;
-					delete_timeout = (uint) ( (int) Preferences.Get (Preferences.EXPORT_EMAIL_DELETE_TIMEOUT_SEC) );
+					delete_timeout = (uint) (Preferences.Get<int> (Preferences.EXPORT_EMAIL_DELETE_TIMEOUT_SEC));
 					delete_timeout = delete_timeout * 1000; // to get milliseconds.
 
 					// Start a timer and when it occurs, delete the temp files.
