@@ -122,25 +122,25 @@ namespace FSpot.Database {
 
 			//Update to version 7.0, keep photo uri instead of path
 			AddUpdate (new Version (7,0), delegate () {
-				MoveTableToTemp ("photos");
+				string tmp_photos = MoveTableToTemp ("photos");
 				ExecuteNonQuery ( 
-					"CREATE TABLE photos (\n" +
-					"	id                 INTEGER PRIMARY KEY NOT NULL,\n" +
-					"       time               INTEGER NOT NULL,\n" +
-					"       uri                STRING NOT NULL,\n" +
-					"       description        TEXT NOT NULL,\n" +
-					"       roll_id            INTEGER NOT NULL,\n" +
+					"CREATE TABLE photos (" +
+					"	id                 INTEGER PRIMARY KEY NOT NULL," +
+					"       time               INTEGER NOT NULL," +
+					"       uri                STRING NOT NULL," +
+					"       description        TEXT NOT NULL," +
+					"       roll_id            INTEGER NOT NULL," +
 					"       default_version_id INTEGER NOT NULL" +
 					")");
-				ExecuteNonQuery (
+				ExecuteNonQuery (String.Format (
 					"INSERT INTO photos (id, time, uri, description, roll_id, default_version_id)	" +
 					"SELECT id, time, 'file://' || directory_path || '/' || name, 		" +
-					"description, roll_id, default_version_id FROM photos_temp ");
+					"description, roll_id, default_version_id FROM {0}", tmp_photos));
 			}, true);
 			
 			// Update to version 8.0, store full version uri
 			AddUpdate (new Version (8,0),delegate () {
-				MoveTableToTemp ("photo_versions");
+				string tmp_versions = MoveTableToTemp ("photo_versions");
 				ExecuteNonQuery (
 					"CREATE TABLE photo_versions (          " +
 					"       photo_id        INTEGER,        " +
@@ -149,9 +149,10 @@ namespace FSpot.Database {
 					"       uri             STRING NOT NULL " +
 					")");
 
-				SqliteDataReader reader = ExecuteReader ("SELECT photo_id, version_id, name, uri " +
-								         "FROM photo_versions_temp, photos " +
-								         "WHERE photo_id = id ");
+				SqliteDataReader reader = ExecuteReader (String.Format (
+						"SELECT photo_id, version_id, name, uri " +
+						"FROM {0}, photos " +
+						"WHERE photo_id = id ", tmp_versions));
 		
 				while (reader.Read ()) {
 					System.Uri photo_uri = new System.Uri (reader [3] as string);
@@ -161,13 +162,14 @@ namespace FSpot.Database {
 					string uri = photo_uri.Scheme + "://" + 
 						photo_uri.Host + 
 						System.IO.Path.GetDirectoryName (photo_uri.AbsolutePath) + "/" +
-						name_without_extension + " (" + (reader [2] as string) + ")" + extension;
+						name_without_extension + " (" + (reader [2]).ToString () + ")" + extension;
+
 					ExecuteNonQuery (new DbCommand (
 						"INSERT INTO photo_versions (photo_id, version_id, name, uri) " +
 						"VALUES (:photo_id, :version_id, :name, :uri)",
 						"photo_id", Convert.ToUInt32 (reader [0]),
 						"version_id", Convert.ToUInt32 (reader [1]),
-						"name", (string)(reader [2]),
+						"name", (reader [2]).ToString (),
 						"uri", uri));
 				}
 
@@ -175,7 +177,7 @@ namespace FSpot.Database {
 			
 			// Update to version 9.0
 			AddUpdate (new Version (9,0),delegate () {
-				MoveTableToTemp ("photo_versions");
+				string tmp_versions = MoveTableToTemp ("photo_versions");
 				ExecuteNonQuery (
 					"CREATE TABLE photo_versions (          " +
 					"       photo_id        INTEGER,        " +
@@ -184,18 +186,166 @@ namespace FSpot.Database {
 					"       uri             STRING NOT NULL," +
 					"	protected	BOOLEAN		" +
 					")");
-				ExecuteNonQuery (
+				ExecuteNonQuery (String.Format (
 					"INSERT INTO photo_versions (photo_id, version_id, name, uri, protected) " +
 					"SELECT photo_id, version_id, name, uri, 0 " +
-					"FROM photo_versions_temp");
+					"FROM {0} ", tmp_versions));
 			});
 
-			// Update to version 10.0
-			//AddUpdate (new Version (0,0),delegate () {
+ 			// Update to version 10.0, make id autoincrement
+ 			AddUpdate (new Version (10,0),delegate () {
+ 				string tmp_photos = MoveTableToTemp ("photos");
+ 				ExecuteNonQuery (
+ 					"CREATE TABLE photos (                                     " +
+ 					"	id                 INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, " +
+ 					"	time               INTEGER NOT NULL,	   	   " +
+ 					"	uri		   STRING NOT NULL,		   " +
+ 					"	description        TEXT NOT NULL,	           " +
+ 					"	roll_id            INTEGER NOT NULL,		   " +
+ 					"	default_version_id INTEGER NOT NULL		   " +
+ 					")");
+ 
+ 				ExecuteNonQuery (String.Format (
+ 					"INSERT INTO photos (id, time, uri, description, roll_id, default_version_id) " +
+ 					"SELECT id, time, uri, description, roll_id, default_version_id  " + 
+ 					"FROM  {0} ", tmp_photos));
+ 			}, false);
+ 
+			// Update to version 11.0, rating
+			AddUpdate (new Version (11,0),delegate () {
+ 				string tmp_photos = MoveTableToTemp ("photos");
+ 				Execute (
+ 					"CREATE TABLE photos (                                     " +
+ 					"	id                 INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, " +
+ 					"	time               INTEGER NOT NULL,	   	   " +
+ 					"	uri		   STRING NOT NULL,		   " +
+ 					"	description        TEXT NOT NULL,	           " +
+ 					"	roll_id            INTEGER NOT NULL,		   " +
+ 					"	default_version_id INTEGER NOT NULL,		   " +
+					"       rating             INTEGER NULL			   " +
+ 					")");
+ 
+ 				Execute (String.Format (
+ 					"INSERT INTO photos (id, time, uri, description, roll_id, default_version_id, rating) " +
+ 					"SELECT id, time, uri, description, roll_id, default_version_id, null  " + 
+ 					"FROM  {0} ", tmp_photos));
+			});
+
+			//Update to version 12.0, remove dead associations, bgo #507950, #488545
+			AddUpdate (new Version (12, 0), delegate () {
+				Execute ("DELETE FROM photo_tags WHERE tag_id NOT IN (SELECT id FROM tags)");
+			});
+			
+			// Update to version 13.0
+			AddUpdate (new Version (13,0), delegate () {
+				Execute ("UPDATE photos SET rating = 0 WHERE rating IS NULL");
+			});
+
+			// Update to version 14.0
+			AddUpdate (new Version (14,0), delegate () {
+				Execute ("UPDATE photos SET rating = 0 WHERE rating IS NULL");
+			});
+
+			// Update to version 15.0
+			AddUpdate (new Version (15,0), delegate () {
+				string tmp_photo_tags = MoveTableToTemp ("photo_tags");
+				Execute (
+					"CREATE TABLE photo_tags (        " +
+					"	photo_id      INTEGER,    " +
+					"       tag_id        INTEGER,    " +
+					"       UNIQUE (photo_id, tag_id) " +
+					")");
+				Execute (String.Format (
+					"INSERT OR IGNORE INTO photo_tags (photo_id, tag_id) " +
+					"SELECT photo_id, tag_id FROM {0}", tmp_photo_tags));
+				string tmp_photo_versions = MoveTableToTemp ("photo_versions");
+				Execute (
+					"CREATE TABLE photo_versions (		"+
+					"	photo_id	INTEGER,	" +
+					"	version_id	INTEGER,	" +
+					"	name		STRING,		" +
+					"	uri		STRING NOT NULL," +
+					"	protected	BOOLEAN, 	" +
+					"	UNIQUE (photo_id, version_id)	" +
+					")");
+				Execute (String.Format (
+					"INSERT OR IGNORE INTO photo_versions 		" +
+					"(photo_id, version_id, name, uri, protected)	" +
+					"SELECT photo_id, version_id, name, uri, protected FROM {0}", tmp_photo_versions));
+			});
+
+			// Update to version 16.0
+			 AddUpdate (new Version (16,0), delegate () {
+				 string temp_table = MoveTableToTemp ("photos");
+  
+				 Execute ("CREATE TABLE photos ( " +
+					  "	id                 INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,   " +
+					  "	time               INTEGER NOT NULL,	   	   " +
+					  "	uri		   STRING NOT NULL,		   " +
+					  "	description        TEXT NOT NULL,	           " +
+					  "	roll_id            INTEGER NOT NULL,		   " +
+					  "	default_version_id INTEGER NOT NULL,		   " +
+					  "	rating		   INTEGER NULL,		   " +
+					  "	md5_sum		   TEXT NULL  			   " +
+					  ")"
+				 );
+  
+				 Execute (string.Format ("INSERT INTO photos (id, time, uri, description, roll_id, " + 
+							 "default_version_id, rating, md5_sum) " + 
+							 "SELECT id, time, uri, description, roll_id, " +
+							 "       default_version_id, rating, '' " +
+							 "FROM   {0} ", 
+							 temp_table
+							)
+				 );
+
+
+				 string temp_versions_table = MoveTableToTemp ("photo_versions");
+
+				 Console.WriteLine("{0} - {1}", temp_table, temp_versions_table);
+
+				 Execute ("CREATE TABLE photo_versions (    	" +
+					  "      photo_id        INTEGER,  	" +
+					  "      version_id      INTEGER,  	" +
+					  "      name            STRING,    	" +
+					  "	uri		STRING NOT NULL," +
+					  "	md5_sum		STRING NOT NULL," +
+					  "	protected	BOOLEAN		" +
+					  ")");
+
+				 Execute (string.Format ("INSERT INTO photo_versions (photo_id, version_id, name, uri, md5_sum, protected) " + 
+							 "SELECT photo_id, version_id, name, uri, '', protected " +
+							 "FROM   {0} ", 
+							 temp_versions_table
+							)
+				 );
+
+				 // This is kind of hacky but should be a lot faster on
+				 // large photo databases
+				 Execute (string.Format ("INSERT INTO jobs (job_type, job_options, run_at, job_priority) " +
+							 "SELECT '{0}', id, {1}, {2} " +
+							 "FROM   photos ",
+							 typeof(Jobs.CalculateHashJob).ToString (),
+							 FSpot.Utils.DbUtils.UnixTimeFromDateTime (DateTime.Now),
+							 0
+							)
+				 );
+			 }, true);
+
+			// Update to version 16.1
+			 AddUpdate (new Version (16,1), delegate () {
+				 Execute ("CREATE INDEX idx_photo_versions_id ON photo_versions(photo_id)");
+			 }, false);
+
+			// Update to version 16.2
+			 AddUpdate (new Version (16,2), delegate () {
+				 Execute ("CREATE INDEX idx_photos_roll_id ON photos(roll_id)");
+			 }, false);
+
+			 // Update to version 17.0
+			//AddUpdate (new Version (14,0),delegate () {
 			//	do update here
 			//});
-
-			
 		}
 
 		public static void Run (Db database)
@@ -285,6 +435,16 @@ namespace FSpot.Database {
 			}
 		}
 
+		private static int Execute (string statement)
+		{
+			return db.Database.Execute (statement);
+		}
+
+		private static int Execute (DbCommand command)
+		{
+			return db.Database.Execute (command);
+		}
+
 		private static void ExecuteNonQuery (string statement)
 		{
 			db.Database.ExecuteNonQuery(statement);
